// mandelbrot.c  from nat-senproj   7 Apr 2012

#include <stdio.h>

#define  mapsize  64
#define  max      256		// max iteration for testing one point

#define  iQ	  	12			// fixed point scale factor, bit
#define  fQ		4096.0

extern int mmap[mapsize][mapsize];

void savefmap(void);

void makemap(void){
	int i, j;
	float x0 = -1.0;
	float y0 = 1.0;
	float x, y;
	float step = 2.0/mapsize;
	float xtemp;
	int iter;

	for(i = 0; i < mapsize; i++){
		x0 = -1.5;
		for(j = 0; j < mapsize; j++){
			x = 0.0;
			y = 0.0;
			iter = 0;
			while( (x*x + y*y < 4.0) && (iter < max)){
				xtemp = x*x - y*y + x0;
				y = 2.0*x*y + y0;
				x = xtemp;
				iter++;
			}
			mmap[j][i] = iter;
			x0 += step;
		}
		y0 -= step;
	}
	savefmap();
}

void savefmap(void){
	FILE *fp;
	int i,j;
	
	fp = fopen("fout.txt","w");
    for(i = 0; i < mapsize; i++){
		for(j = 0; j < mapsize; j++){
			fprintf(fp,"%d ",mmap[i][j]);
		}
		fprintf(fp,"\n");
	}
	fclose(fp);
}

int ibound = (int)(4.0*fQ);

int testbound(int x, int y){
	int tx, ty;
	tx = (x*x) >> iQ;
	ty = (y*y) >> iQ;
	return (tx + ty) < ibound;
}

// x*x - y*y + x0
int nextx(int x, int y, int x0){
	int tx, ty;
	tx = (x*x) >> iQ;
	ty = (y*y) >> iQ;
	return (tx - ty + x0);
}
// 2*x*y + y0
int nexty(int x, int y, int y0){
	int t, t2;
	t = (x*y) >> iQ;
	t2 = ((2 << iQ) * t) >> iQ;
	return t2 + y0;
}

void saveparam(int size, int q, int x0, int y0, int step){
	FILE *fp;
	fp = fopen("param.txt","w");
	fprintf(fp,"mapsize %d Q %d x0 %d y0 %d step %d bound %d\n",
		size,q,x0,y0,step,ibound);
	fclose(fp);
}

void makemap2(void){

	int i, j;
	float x0 = -1.5;
	float y0 = 1.0;
	float x, y;
	float step = 2.0/mapsize;
	float xtemp;
	int iter;

	int ix0 = (int)(x0*fQ);
	int tx0 = ix0;					// temp
	int iy0 = (int)(y0*fQ);
	int ix, iy, xt;
	int istep = (int)(step*fQ);

	saveparam(mapsize,iQ,ix0,iy0,istep);

	for(i = 0; i < mapsize; i++){
		ix0 = tx0;
		for(j = 0; j < mapsize; j++){
			ix = 0;
			iy = 0;
			iter = 0;
			while( testbound(ix,iy) && (iter < max)){
				xt = nextx(ix,iy,ix0);
				iy = nexty(ix,iy,iy0);
				ix = xt;
				iter++;
			}
			mmap[j][i] = iter;
			ix0 += istep;
		}
		iy0 -= istep;
	}
	savefmap();
}


